@@ -1,30 +1,39 @@
-/* cxxtools/log_cxxtools.h
- *
- * cxxtools - general purpose C++-toolbox
+/*
  * Copyright (C) 2003, Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 #ifndef CXXTOOLS_LOG_CXXTOOLS_H
 #define CXXTOOLS_LOG_CXXTOOLS_H
 
 #include <string>
-#include <cxxtools/thread.h>
+#include <cxxtools/mutex.h>
 #include <iostream>
+#include <sstream>
+#include <cxxtools/api.h>
 
 #define log_xxxx_enabled(level)   \
   (getLogger() != 0 && getLogger()->isEnabled(::cxxtools::Logger::LOG_LEVEL_ ## level))
@@ -61,11 +70,13 @@
   }
 
 #define log_define(category) \
-  static inline ::cxxtools::Logger* getLogger()   \
+  static ::cxxtools::Logger* getLogger()   \
   {  \
     static cxxtools::Logger* logger = 0; \
+    if (!::cxxtools::Logger::isEnabled()) \
+      return 0; \
     if (logger == 0) \
-      logger = ::cxxtools::Logger::getLogger(category); \
+      logger = ::cxxtools::Logger::getCategoryLogger(category); \
     return logger; \
   }
 
@@ -96,7 +107,7 @@ namespace cxxtools
       virtual ~Logger()  { }
 
     public:
-      static Logger* getLogger(const std::string& category);
+      static Logger* getCategoryLogger(const std::string& category);
       static void setRootLevel(log_level_type l)
         { std_level = l; }
       static log_level_type getStdLevel()
@@ -114,10 +125,13 @@ namespace cxxtools
         { return level; }
       void setLogLevel(log_level_type l)
         { level = l; }
+      /// prints the start of a longentry into the outstream
+      static std::ostream& logentry(std::ostream& out, const char* level, const std::string& category);
+      /// prints the start of a longentry and returns the appender
       std::ostream& logentry(const char* level);
       virtual void logEnd(std::ostream& appender) = 0;
 
-      static RWLock rwmutex;
+      static ReadWriteMutex rwmutex;
       static Mutex mutex;
   };
 
@@ -128,6 +142,7 @@ namespace cxxtools
 
     public:
       LogMessage(Logger* logger, const char* level);
+      LogMessage(Logger* logger, Logger::log_level_type level);
       ~LogMessage();
 
       std::ostream& out();
