@@ -1,27 +1,37 @@
 /*
  * Copyright (C) 2006 Tommi Maekitalo
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * is provided AS IS, WITHOUT ANY WARRANTY; without even the implied
- * warranty of MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and
- * NON-INFRINGEMENT.  See the GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
- *
+ * 
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #include <cxxtools/fdstream.h>
-#include <cxxtools/syserror.h>
+#include <cxxtools/systemerror.h>
+#include <cxxtools/log.h>
 #include <algorithm>
 #include <unistd.h>
-#include <cxxtools/log.h>
+#include <errno.h>
 
 log_define("cxxtools.fdstream")
 
@@ -58,15 +68,18 @@ namespace cxxtools
     {
       log_debug("write " << (pptr() - pbase()) << " bytes to fd " << fd);
       ssize_t ret = ::write(fd, pbase(), pptr() - pbase());
-      if (ret < 0)
-        throw SysError("write");
-      else if (ret == 0)
+
+      if(ret < 0)
+        throw SystemError(errno, "write");
+
+      if (ret == 0)
         return traits_type::eof();
       else
       {
         log_debug(ret << " bytes written to fd " << fd);
         std::copy(pptr(), pptr() + ret, obuffer);
-        setp(obuffer + ret, obuffer + bufsize - ret);
+        setp(obuffer, obuffer + bufsize - ret);
+        pbump(ret);
       }
     }
     else
@@ -103,15 +116,17 @@ namespace cxxtools
 
     log_debug("read from fd " << fd);
     int ret = ::read(fd, ibuffer, bufsize);
-    if (ret < 0)
-      throw SysError("read");
-    else if (ret == 0)
+
+    if(ret < 0)
+      throw SystemError(errno, "read");
+
+    if (ret == 0)
       return traits_type::eof();
 
     log_debug(ret << " bytes read");
     setg(ibuffer, ibuffer, ibuffer + ret);
 
-    return traits_type::to_char_type(*gptr());
+    return traits_type::to_int_type(*gptr());
   }
 
   int Fdstreambuf::sync()
@@ -124,9 +139,11 @@ namespace cxxtools
       {
         log_debug("write " << (pptr() - p) << " bytes to fd " << fd);
         ssize_t ret = ::write(fd, p, pptr() - p);
-        if (ret < 0)
-          throw SysError("write");
-        else if (ret == 0)
+
+        if(ret < 0)
+          throw SystemError(errno, "write");
+
+        if (ret == 0)
           return traits_type::eof();
 
         log_debug(ret << " bytes written to fd " << fd);
