@@ -1,31 +1,39 @@
-/* log.cpp
- *
- * cxxtools - general purpose C++-toolbox
+/*
  * Copyright (C) 2003,2004 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 #include <cxxtools/log/cxxtools_init.h>
-#include <cxxtools/thread.h>
-#include <cxxtools/udp.h>
-#include <cxxtools/udpstream.h>
+#include <cxxtools/mutex.h>
+#include <cxxtools/net/udp.h>
+#include <cxxtools/net/udpstream.h>
 #include <cxxtools/tee.h>
 #include <cxxtools/streamcounter.h>
-#include <cxxtools/pipestream.h>
+#include <cxxtools/posix/pipestream.h>
+#include <cxxtools/systemerror.h>
 #include <list>
 #include <vector>
 #include <algorithm>
@@ -33,6 +41,7 @@
 #include <sstream>
 #include <locale>
 #include <time.h>
+#include <pthread.h>
 #include <sys/time.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -80,13 +89,13 @@ namespace cxxtools
   {
       static std::string fname;
       static std::ofstream outfile;
-      static cxxtools::Streamcounter counter;
-      static cxxtools::Tee tee;
+      static Streamcounter counter;
+      static Tee tee;
       static net::UdpSender loghost;
       static net::UdpOStream udpmessage;
       static unsigned maxfilesize;
       static unsigned maxbackupindex;
-      static Pipestream* pipe;
+      static posix::Pipestream* pipe;
 
       static std::string mkfilename(unsigned idx);
 
@@ -173,13 +182,13 @@ namespace cxxtools
 
   std::string LoggerImpl::fname;
   std::ofstream LoggerImpl::outfile;
-  cxxtools::Streamcounter LoggerImpl::counter;
-  cxxtools::Tee LoggerImpl::tee(LoggerImpl::outfile, LoggerImpl::counter);
+  Streamcounter LoggerImpl::counter;
+  Tee LoggerImpl::tee(LoggerImpl::outfile, LoggerImpl::counter);
   net::UdpSender LoggerImpl::loghost;
   net::UdpOStream LoggerImpl::udpmessage(LoggerImpl::loghost);
   unsigned LoggerImpl::maxfilesize = 0;
   unsigned LoggerImpl::maxbackupindex = 0;
-  Pipestream* LoggerImpl::pipe = 0;
+  posix::Pipestream* LoggerImpl::pipe = 0;
 
   void LoggerImpl::setFile(const std::string& fname_)
   {
@@ -213,10 +222,12 @@ namespace cxxtools
         throw std::runtime_error("unknown group \"" + group + "\" in logging configuration");
     }
 
-    pipe = new Pipestream();
+    pipe = new posix::Pipestream();
     pid_t pid = ::fork();
-    if (pid < 0)
-      throw SysError("fork");
+
+    if(pid < 0)
+      throw SystemError("fork");
+
     if (pid == 0)
     {
       // 1st child
@@ -259,13 +270,24 @@ namespace cxxtools
     }
   }
 
-  RWLock Logger::rwmutex;
+  ReadWriteMutex Logger::rwmutex;
   Mutex Logger::mutex;
   Logger::log_level_type Logger::std_level = LOG_LEVEL_ERROR;
   bool Logger::enabled = false;
 
   namespace
   {
+    class StaticDeinitializer
+    {
+      public:
+        ~StaticDeinitializer()
+        {
+          Logger::setEnabled(false);
+        }
+    };
+
+    StaticDeinitializer staticDeinitializer;
+
     typedef std::list<Logger*> loggers_type;
 
     loggers_type& getBaseLoggers()
@@ -336,7 +358,7 @@ namespace cxxtools
       return 0;
 
     // search existing Logger
-    RdLock rdLock(rwmutex);
+    ReadLock rdLock(rwmutex);
 
     loggers_type::iterator lower_bound_it = getCacheLoggers().begin();
     while (lower_bound_it != getCacheLoggers().end()
@@ -349,7 +371,7 @@ namespace cxxtools
 
     // Logger not in list - change to write-lock
     rdLock.unlock();
-    WrLock wrLock(rwmutex);
+    WriteLock wrLock(rwmutex);
 
     // we have to do it again after gaining write-lock
     lower_bound_it = getCacheLoggers().begin();
@@ -371,7 +393,7 @@ namespace cxxtools
 
   Logger* Logger::setLevel(const std::string& category, log_level_type l)
   {
-    WrLock lock(rwmutex);
+    WriteLock lock(rwmutex);
 
     // search for existing Logger
     loggers_type& baseLoggers = getBaseLoggers();
@@ -474,12 +496,21 @@ namespace cxxtools
       std::ostream& out()     { return msg; }
       void flush()
       {
-        MutexLock lock(Logger::mutex);
+        if (!Logger::isEnabled())
+          return;
 
-        std::ostream& out(logger->logentry(level));
-        out << msg.str() << '\n';
+        try
+        {
+          MutexLock lock(Logger::mutex);
 
-        logger->logEnd(out);
+          std::ostream& out(logger->logentry(level));
+          out << msg.str() << '\n';
+
+          logger->logEnd(out);
+        }
+        catch (const std::exception&)
+        {
+        }
       }
 
   };
@@ -516,11 +547,18 @@ namespace cxxtools
   {
     if (msg)
     {
-      if (l->isEnabled(Logger::LOG_LEVEL_TRACE))
+      if (Logger::isEnabled()
+        && l->isEnabled(Logger::LOG_LEVEL_TRACE))
       {
-        cxxtools::MutexLock lock(cxxtools::Logger::mutex);
-        l->logentry("TRACE")
-          << "EXIT " << msg->str() << std::endl;
+        try
+        {
+          MutexLock lock(Logger::mutex);
+          l->logentry("TRACE")
+            << "EXIT " << msg->str() << std::endl;
+        }
+        catch (const std::exception&)
+        {
+        }
       }
       delete msg;
     }
@@ -535,11 +573,19 @@ namespace cxxtools
 
   void LogTracer::enter()
   {
-    if (msg && l->isEnabled(Logger::LOG_LEVEL_TRACE))
+    if (msg
+      && Logger::isEnabled()
+      && l->isEnabled(Logger::LOG_LEVEL_TRACE))
     {
-      cxxtools::MutexLock lock(cxxtools::Logger::mutex);
-      l->logentry("TRACE")
-        << "ENTER " << msg->str() << std::endl;
+      try
+      {
+        MutexLock lock(Logger::mutex);
+        l->logentry("TRACE")
+          << "ENTER " << msg->str() << std::endl;
+      }
+      catch (const std::exception&)
+      {
+      }
     }
   }
 
